/* $Id$ */
/*
 * Cantus Tag Editor
 * Copyright © 2002-2004 by Samuel Abels <spam@debain.org>
 * Copyright © 2007 by Tim Huetz <tim.huetz@futures-net.de>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 **/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifndef HAVE_EDITAREA_H
#define HAVE_EDITAREA_H

#include <iostream>
#include <gtkmm.h>
#include <libintl.h>
#include <list>
#include <cantus/gnuhashmap.h>

#define _(String) gettext (String)
#define gettext_noop(String) (String)
#define N_(String) gettext_noop (String)

using namespace std;
using namespace Gtk;

enum SOURCETAG {
  SOURCETAG_ID3V1,
  SOURCETAG_ID3V2,
  SOURCETAG_OGG,
  SOURCETAG_END
};

static const char *tagname[] = {
  "ID3V1",
  "ID3V2",
  "OGG"
};


class Editarea : public sigc::trackable {
public:
  Editarea();
  ~Editarea();
  
  /* Triggered whenever the save button has been clicked. */
  SigC::Signal0<void> signal_button_save_clicked;
  /* Triggered whenever the copy direction has been changed. */
  SigC::Signal0<void> signal_sourcetag_changed;
  /* Triggered whenever "format" field has been changed. */
  SigC::Signal0<void> signal_format_changed;
  
  GtkWidget     *build(bool vertical);
  GtkWidget     *build_horizontal(void);
  GtkWidget     *build_vertical(void);
  void           set_active(bool active);
  
  Gtk::Widget   *get_widget(const gchar *widgetname);
  void           set_label_text(const gchar *widgetname,
                                const gchar *str,
                                bool error = FALSE);
  Glib::ustring  get_entry_text(const gchar *widgetname);
  void           append_combo_entry_text(const gchar *widgetname,
                                          std::string text);
  Glib::ustring  get_combo_entry_text(const gchar *widgetname);
  gboolean       get_check_active(const gchar *widgetname);
  int            get_sourcetag(void);
  
private:
  Gtk::VBox  *build_form(void);
  Gtk::Frame *build_legend(void);
  void build_buttonbox(bool vertical);
  
  std::list<long>    listenerids;       // The ids of the eventbus listeners.
                                        // A hashtable holding all widgetnames.
  __gnu_cxx::hash_map<std::string, Gtk::Widget*> widgets;
  std::list<Gtk::Widget*> focuschain;   // A list holding the focus order.
  Gtk::Tooltips tooltips;               // Manages all tooltips.
  Gtk::Box      *parent;                // The table holding all widgets.
  Gtk::Box      *buttonbox;             // The box holding all buttons.
};

#endif
