/* $Id$ */
/*
 * Cantus Tag Editor
 * Copyright © 2002-2004 by Samuel Abels <spam@debain.org>
 * Copyright © 2007 by Tim Huetz <tim.huetz@futures-net.de>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 **/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <cantusplugins/tag2filename/editarea.h>

//#define _DEBUG_


/******************************************************************************
 * Constructor/Destructor
 ******************************************************************************/
Editarea::Editarea()
{
}


Editarea::~Editarea(void)
{
#ifdef _DEBUG_
  printf("Editarea::~Editarea(): Called.\n");
#endif
}


/******************************************************************************
 * Public
 ******************************************************************************/
GtkWidget* Editarea::build_horizontal(void)
{
  HPaned* toplevel  = new HPaned;
  VBox*   form      = build_form();
  Frame*  legend    = build_legend();
  parent = new HBox;
  
  toplevel->set_position(380);
  form->set_border_width(6);
  legend->set_border_width(6);
  
  toplevel->add1(*manage(form));
  toplevel->add2(*manage(legend));
  toplevel->set_focus_chain(focuschain);
  
  parent->pack_start(*manage(toplevel), TRUE, TRUE);
  parent->pack_start(*manage(buttonbox), FALSE, TRUE);
  parent->show_all();
  
  return (GtkWidget*)Glib::unwrap(parent);
}


GtkWidget* Editarea::build_vertical(void)
{
  VBox*  form   = build_form();
  Frame* legend = build_legend();
  parent = new VBox;
  
  parent->set_border_width(6);
  parent->pack_start(*manage(form), FALSE, TRUE);
  parent->pack_start(*manage(legend), TRUE, TRUE);
  parent->pack_start(*manage(buttonbox), FALSE, TRUE);
  parent->set_focus_chain(focuschain);
  parent->show_all();
  
  return (GtkWidget*)Glib::unwrap(parent);
}


GtkWidget* Editarea::build(bool vertical)
{
  build_buttonbox(vertical);
  if (vertical)
    return build_vertical();
  else
    return build_horizontal();
}


Gtk::Widget *Editarea::get_widget(const gchar *widgetname)
{
  Gtk::Widget *widget = widgets[widgetname];
  if (!widget)
    g_warning("Editarea::get_widget(): Widget not found: %s\n", widgetname);
  g_assert(widget != NULL);
  return widget;
}


void Editarea::set_label_text(const gchar *widgetname,
                              const gchar *str,
                              bool error)
{
#ifdef _DEBUG_
  printf("Editarea::~set_label_text(): Called. (%s)\n", str);
#endif
  Label *label = (Label*)get_widget(widgetname);
  if (error) {
    std::string text;
    text.append("<span color='red'>");
    text.append(str);
    text.append("</span>");
    label->set_markup(text);
  }
  else
    label->set_text(str ? str : "");
}


Glib::ustring Editarea::get_entry_text(const gchar *widgetname)
{
  Gtk::Entry *entry = (Gtk::Entry*)get_widget(widgetname);
  return entry->get_text();
}


void Editarea::append_combo_entry_text(const gchar *widgetname,
                                        std::string text)
{
  Gtk::ComboBoxEntryText *cmb = (Gtk::ComboBoxEntryText*)get_widget(widgetname);
  cmb->append_text(text);
}


Glib::ustring Editarea::get_combo_entry_text(const gchar *widgetname)
{
  Gtk::ComboBoxEntryText *cmb = (Gtk::ComboBoxEntryText*)get_widget(widgetname);
  Gtk::Entry             *entry = (Gtk::Entry*)cmb->get_child();
  return entry->get_text();
}


gboolean Editarea::get_check_active(const gchar *widgetname)
{
  Gtk::CheckButton *check = (Gtk::CheckButton*)get_widget(widgetname);
  return check->get_active();
}


void Editarea::set_active(bool active)
{
  parent->set_sensitive(active);
}


int Editarea::get_sourcetag(void)
{
  ComboBoxText* combo = (ComboBoxText*)widgets["SourceTag"];
  return combo->get_active_row_number();
}


/******************************************************************************
 * Private
 ******************************************************************************/
Gtk::VBox *Editarea::build_form(void)
{
  Gtk::VBox              *topvbox   = new Gtk::VBox;
  Gtk::HBox              *tophbox   = new Gtk::HBox;
  Gtk::VBox              *leftvbox  = new Gtk::VBox;
  Gtk::VBox              *rightvbox = new Gtk::VBox;
  Gtk::Label             *label     = NULL;
  Gtk::Entry             *entry     = NULL;
  Gtk::ComboBoxText      *combo     = NULL;
  Gtk::ComboBoxEntryText *combo_e   = NULL;
  Gtk::ScrolledWindow    *scrolled  = NULL;
  Gtk::Viewport          *view      = NULL;
  Gtk::RadioButton       *radio     = NULL;
  Gtk::RadioButton::Group group;
  
  topvbox->pack_start(*Gtk::manage(tophbox), FALSE, FALSE);
  label = new Gtk::Label;
  topvbox->pack_start(*Gtk::manage(label), TRUE, TRUE);
  
  leftvbox->set_homogeneous();
  leftvbox->set_spacing(6);
  rightvbox->set_homogeneous();
  rightvbox->set_spacing(6);
  tophbox->set_spacing(12);
  tophbox->pack_start(*Gtk::manage(leftvbox), FALSE, FALSE);
  tophbox->pack_start(*Gtk::manage(rightvbox), TRUE, TRUE);
  
  // Fill the left column.
  label = new Gtk::Label(_("Source Tag:"), 0, 0.5);
  leftvbox->pack_start(*Gtk::manage(label));
  
  label = new Gtk::Label(_("Format:"), 0, 0.5);
  leftvbox->pack_start(*Gtk::manage(label));
  
  label = new Gtk::Label(_("Filename:"), 0, 0.5);
  leftvbox->pack_start(*Gtk::manage(label));
  
  // Fill the right column.
  combo = new Gtk::ComboBoxText;
  combo->insert_text(SOURCETAG_ID3V1, _("ID3 Tag Version 1"));
  combo->insert_text(SOURCETAG_ID3V2, _("ID3 Tag Version 2"));
  combo->insert_text(SOURCETAG_OGG,   _("OGG Tag"));
  combo->set_active(SOURCETAG_ID3V2);
  combo->signal_changed().connect(signal_sourcetag_changed);
  widgets["SourceTag"] = combo;
  rightvbox->pack_start(*Gtk::manage(combo));
  
  combo_e = new Gtk::ComboBoxEntryText;
  entry = (Gtk::Entry*)combo_e->get_child();
  entry->signal_changed().connect(signal_format_changed);
  widgets["Format"] = combo_e;
  focuschain.push_back(combo_e);
  rightvbox->pack_start(*Gtk::manage(combo_e));
  
  scrolled = new Gtk::ScrolledWindow;
  label    = new Gtk::Label("", 0, 0.5);
  widgets["Filename"] = label;
  scrolled->set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_NEVER);
  scrolled->add(*Gtk::manage(label));
  view = (Gtk::Viewport*)scrolled->get_child();
  view->set_shadow_type(Gtk::SHADOW_NONE);
  rightvbox->pack_start(*Gtk::manage(scrolled), FALSE, TRUE);
  
  return topvbox;
}


Gtk::Frame *Editarea::build_legend(void)
{
  Gtk::Label          *label    = NULL;
  Gtk::Frame          *frame    = new Gtk::Frame;
  Gtk::Table          *table    = new Gtk::Table(6, 3);
  Gtk::ScrolledWindow *scrolled = NULL;
  Gtk::Viewport       *view     = NULL;
  
  std::string text;
  text.append("<b>");
  text.append(_("Tag Fields"));
  text.append("</b>");
  label = new Gtk::Label;
  label->set_markup(text);
  frame->set_label_widget(*Gtk::manage(label));
  frame->set_shadow_type(Gtk::SHADOW_NONE);
  frame->add(*Gtk::manage(table));
  table->set_col_spacing(1, 12);
  table->set_border_width(3);
  
  // Add the description labels.
  label = new Gtk::Label(_("Artist (%a)"), 1, 0.5);
  table->attach(*Gtk::manage(label), 1, 2, 0, 1, Gtk::FILL, Gtk::FILL);
  
  label = new Gtk::Label(_("Song (%s)"), 1, 0.5);
  table->attach(*Gtk::manage(label), 1, 2, 1, 2, Gtk::FILL, Gtk::FILL);
  
  label = new Gtk::Label(_("Album (%b)"), 1, 0.5);
  table->attach(*Gtk::manage(label), 1, 2, 2, 3, Gtk::FILL, Gtk::FILL);
  
  label = new Gtk::Label(_("Track (%t/%T)"), 1, 0.5);
  tooltips.set_tip(*Gtk::manage(label),
                   _("Use %t for one-digit track numbers "
                     "or %T for two-digit track numbers."));
  table->attach(*Gtk::manage(label), 1, 2, 3, 4, Gtk::FILL, Gtk::FILL);
  
  label = new Gtk::Label(_("Year (%y)"), 1, 0.5);
  table->attach(*Gtk::manage(label), 1, 2, 4, 5, Gtk::FILL, Gtk::FILL);
  
  label = new Gtk::Label(_("Genre (%g)"), 1, 0.5);
  table->attach(*Gtk::manage(label), 1, 2, 5, 6, Gtk::FILL, Gtk::FILL);
  
  // Add the content labels. Each of them is in in a scrolled window.
  scrolled = new Gtk::ScrolledWindow;
  label    = new Gtk::Label("", 0, 0.5);
  label->set_selectable();
  widgets["Artist"] = label;
  scrolled->set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_NEVER);
  scrolled->add(*Gtk::manage(label));
  view = (Gtk::Viewport*)scrolled->get_child();
  view->set_shadow_type(Gtk::SHADOW_NONE);
  table->attach(*Gtk::manage(scrolled), 2, 3, 0, 1, Gtk::FILL|Gtk::EXPAND,
                                                    Gtk::FILL);
  
  scrolled = new Gtk::ScrolledWindow;
  label    = new Gtk::Label("", 0, 0.5);
  label->set_selectable();
  widgets["Song"] = label;
  scrolled->set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_NEVER);
  scrolled->add(*Gtk::manage(label));
  view = (Gtk::Viewport*)scrolled->get_child();
  view->set_shadow_type(Gtk::SHADOW_NONE);
  table->attach(*Gtk::manage(scrolled), 2, 3, 1, 2, Gtk::FILL|Gtk::EXPAND,
                                                    Gtk::FILL);
  
  scrolled = new Gtk::ScrolledWindow;
  label    = new Gtk::Label("", 0, 0.5);
  label->set_selectable();
  widgets["Album"] = label;
  scrolled->set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_NEVER);
  scrolled->add(*Gtk::manage(label));
  view = (Gtk::Viewport*)scrolled->get_child();
  view->set_shadow_type(Gtk::SHADOW_NONE);
  table->attach(*Gtk::manage(scrolled), 2, 3, 2, 3, Gtk::FILL|Gtk::EXPAND,
                                                    Gtk::FILL);
  
  scrolled = new Gtk::ScrolledWindow;
  label    = new Gtk::Label("", 0, 0.5);
  label->set_selectable();
  widgets["Track"] = label;
  scrolled->set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_NEVER);
  scrolled->add(*Gtk::manage(label));
  view = (Gtk::Viewport*)scrolled->get_child();
  view->set_shadow_type(Gtk::SHADOW_NONE);
  table->attach(*Gtk::manage(scrolled), 2, 3, 3, 4, Gtk::FILL|Gtk::EXPAND,
                                                    Gtk::FILL);
  
  scrolled = new Gtk::ScrolledWindow;
  label    = new Gtk::Label("", 0, 0.5);
  label->set_selectable();
  widgets["Year"] = label;
  scrolled->set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_NEVER);
  scrolled->add(*Gtk::manage(label));
  view = (Gtk::Viewport*)scrolled->get_child();
  view->set_shadow_type(Gtk::SHADOW_NONE);
  table->attach(*Gtk::manage(scrolled), 2, 3, 4, 5, Gtk::FILL|Gtk::EXPAND,
                                                    Gtk::FILL);
  
  scrolled = new Gtk::ScrolledWindow;
  label    = new Gtk::Label("", 0, 0.5);
  label->set_selectable();
  widgets["Genre"] = label;
  scrolled->set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_NEVER);
  scrolled->add(*Gtk::manage(label));
  view = (Gtk::Viewport*)scrolled->get_child();
  view->set_shadow_type(Gtk::SHADOW_NONE);
  table->attach(*Gtk::manage(scrolled), 2, 3, 5, 6, Gtk::FILL|Gtk::EXPAND,
                                                    Gtk::FILL);
  
  return frame;
}


void Editarea::build_buttonbox(bool vertical)
{
  Button* button = NULL;
  
  if (vertical)
    buttonbox = new HBox;
  else
    buttonbox = new VBox;
  
  buttonbox->set_spacing(6);
  buttonbox->set_border_width(6);
  
  Fixed* fixed = new Fixed;
  if (vertical)
    buttonbox->pack_start(*manage(fixed), TRUE, TRUE);
  else
    buttonbox->pack_end(*manage(fixed), TRUE, TRUE);
  
  // Add the "Save" button.
  button = new Button(Stock::SAVE);
  button->set_size_request(-1, 35);
  if (vertical)
    buttonbox->pack_end(*manage(button), FALSE, TRUE);
  else
    buttonbox->pack_start(*manage(button), FALSE, TRUE);
  button->signal_clicked().connect(signal_button_save_clicked);
}
